//  Douglas M. Hurst
//  8912 Spur Road
//  Springfield, VA 22153
//  CompuServe 72767,1163
//
//  BAR39  - is a complete program to produce 3 of 9 Bar Codes
//           on an HP LaserJet Plus (or newer) or compatible printer,
//           WITHOUT THE REQUIREMENT OF AN EXTERNAL LIBRARY OR CARTRIDGE.           
//           This program takes as its arguments a string of characters
//           that are part of the 3 of 9 character set.  You are 
//           responsible for formatting the string prior to sending 
//           it off to the hp_printbar39() function.  If you include
//           non-bar39 characters, you will hear about it from that
//           function.
//
//                        Example: *EAC9217250001*
//
//           This program is not really intended to be used as is.  Just
//           use its pieces as you will to make it work for you.
//
#include <stdio.h>
#include <string.h>

#define HP_INIT   "\33&l%dO\33(s0p10h12v0s0b3T"
#define HP_DEINIT "\33E"

#define LANDSCAPE     1
#define PORTRAIT      0
#define BAR           "\33*p%dx%dY\33*c%da%db%dP"
#define WIDE          9
#define NARROW        3
#define TALL        180
#define BLACK         0
#define WHITE         1
#define CPI          10
#define RPI           6
#define DENSITY   300.0

void main(int, char **);
int  hp_printbar39(int, int, char *);
void hp_gotoxy(int, int, int *, int *);

void main(int argc, char **argv)
{
  int i,j,k;

  argc--;
  if (!argc)
  {
    printf("USAGE: BAR39 <code string>\n");
    exit(1);
  }
  argv++;

  fprintf(stdprn,HP_INIT,PORTRAIT);

  if (hp_printbar39(25,30,*argv))
  {
    printf("Non-Bar39 character in string.\n");
    exit(1);
  }

  fprintf(stdprn,HP_DEINIT);
}

// H P _ P R I N T _ B A R
//
int hp_printbar39(int col, int row, char *code)
{
  char jstr[11];
  int i,j,pointx,pointy;

  hp_gotoxy(col,row, &pointx, &pointy);

  for (i=0 ; code[i] ; i++)
  {
    switch(code[i])
    {
      case '$': strcpy(jstr,"-1-1-1----"); break;
      case '%': strcpy(jstr,"---1-1-1--"); break;
      case '*': strcpy(jstr,"-1--1-1---"); break;
      case '+': strcpy(jstr,"-1---1-1--"); break;
      case ',': strcpy(jstr,"-11---1---"); break;
      case '-': strcpy(jstr,"-1----1-1-"); break;
      case '.': strcpy(jstr,"11----1---"); break;
      case '/': strcpy(jstr,"-1-1---1--"); break;
      case '0': strcpy(jstr,"---11-1---"); break;
      case '1': strcpy(jstr,"1--1----1-"); break;
      case '2': strcpy(jstr,"--11----1-"); break;
      case '3': strcpy(jstr,"1-11------"); break;
      case '4': strcpy(jstr,"---11---1-"); break;
      case '5': strcpy(jstr,"1--11-----"); break;
      case '6': strcpy(jstr,"--111-----"); break;
      case '7': strcpy(jstr,"---1--1-1-"); break;
      case '8': strcpy(jstr,"1--1--1---"); break;
      case '9': strcpy(jstr,"--11--1---"); break;
      case 'A': strcpy(jstr,"1----1--1-"); break;
      case 'B': strcpy(jstr,"--1--1--1-"); break;
      case 'C': strcpy(jstr,"1-1--1----"); break;
      case 'D': strcpy(jstr,"----11--1-"); break;
      case 'E': strcpy(jstr,"1---11----"); break;
      case 'F': strcpy(jstr,"--1-11----"); break;
      case 'G': strcpy(jstr,"-----11-1-"); break;
      case 'H': strcpy(jstr,"1----11---"); break;
      case 'I': strcpy(jstr,"--1--11---"); break;
      case 'J': strcpy(jstr,"----111---"); break;
      case 'K': strcpy(jstr,"1------11-"); break;
      case 'L': strcpy(jstr,"--1----11-"); break;
      case 'M': strcpy(jstr,"1-1----1--"); break;
      case 'N': strcpy(jstr,"----1--11-"); break;
      case 'O': strcpy(jstr,"1---1--1--"); break;
      case 'P': strcpy(jstr,"--1-1--1--"); break;
      case 'Q': strcpy(jstr,"------111-"); break;
      case 'R': strcpy(jstr,"1-----11--"); break;
      case 'S': strcpy(jstr,"--1---11--"); break;
      case 'T': strcpy(jstr,"----1-11--"); break;
      case 'U': strcpy(jstr,"11------1-"); break;
      case 'V': strcpy(jstr,"-11-----1-"); break;
      case 'W': strcpy(jstr,"111-------"); break;
      case 'X': strcpy(jstr,"-1--1---1-"); break;
      case 'Y': strcpy(jstr,"11--1-----"); break;
      case 'Z': strcpy(jstr,"-11-1-----"); break;
      default:  return(1);
    }
    for (j=0 ; jstr[j] ; j++)
    {
      if (j % 2 == 0)
      {
        if (jstr[j] == '1')
        {
          fprintf(stdprn,BAR,pointx,pointy,WIDE,TALL,BLACK);
          pointx+=9;
        }
        else
        {
          fprintf(stdprn,BAR,pointx,pointy,NARROW,TALL,BLACK);
          pointx+=3;
        }
      }
      else
      {
        if (jstr[j] == '1')
        {
          fprintf(stdprn,BAR,pointx,pointy,WIDE,TALL,WHITE);
          pointx+=9;
        }
        else
        {
          fprintf(stdprn,BAR,pointx,pointy,NARROW,TALL,WHITE);
          pointx+=3;
        }
      }
    }
  } 
  return(0);
}

// H P _ G O T O X Y
//
void hp_gotoxy(int col, int row, int *px, int *py)
{
  fprintf(stdprn,"\33(8U\33(s0p%dh3T",CPI);
  fprintf(stdprn,"\33&a%dc%dR",col,row);
  *px = (int)((float)col/(float)CPI*DENSITY);
  *py = (int)((float)row/(float)RPI*DENSITY);
  fprintf(stdprn,"\33*p%dx%dY",*px,*py);
}

