//  Douglas M. Hurst
//  8912 Spur Road
//  Springfield, VA 22153
//  CompuServe 72767,1163
//
//  BARZIP - is a complete program to produce U.S. Postal Bar Codes
//           on an HP LaserJet Plus (or newer) or compatible printer,
//           WITHOUT THE REQUIREMENT OF AN EXTERNAL LIBRARY OR CARTRIDGE.           
//           This program takes as its arguments a zip code of 
//           (5 or 9) digit, and address line simulating a street
//           address, a rural route or a post office box.  If the 
//           zip code is a 5 digit code, the address is ignored.
//           If the zip is a 9 digit code, the address line is parsed
//           to get up to the last two of the first set of numbers 
//           encountered.  
//
//                        Examples:  8912 Spur Road  (12)
//                                   1 Curie Court   (01)
//                                   Route 1, Box 12 (01)
//                                   P.O. Box 2345   (45)
//
//           The function add_adv_zip_check_digit() takes the raw 
//           zip_code and turns it into a code that can be passed
//           to the printing function hp_printbarzip().  hp_printbarzip()
//           translates all the characters to their representative 
//           bar code and prints them.
//
//           This program is not really intended to be used as is.  Just
//           use its pieces as you will to make it work for you.
//
#include <stdio.h>
#include <string.h>

#define HP_INIT   "\33&l%dO\33(s0p10h12v0s0b3T"
#define HP_DEINIT "\33E"

#define LANDSCAPE     1
#define PORTRAIT      0
#define BAR           "\33*p%dx%dY\33*c%da%db%dP"
#define WIDE          8
#define NARROW        5
#define TALL         36
#define SHORT_START  20
#define SHORT        16
#define BLACK         0
#define WHITE         1
#define CPI          10
#define RPI           6
#define DENSITY   300.0

void main(int, char **);
int  hp_printbarzip(int, int, char *);
void hp_gotoxy(int, int, int *, int *);
int  add_adv_ZIP_check_digit(char *, char *,char *);

void main(int argc, char **argv)
{
  char orig_zip[10], data_buf[14];
  int i,j,k;

  if (argc < 3)
  {
    printf("\nUSAGE: barzip <zip code string> <street address (enclose in \"'s)\n\n");
    printf("Example: barzip 221531243 \"8912 Spur Road\"\n");
    exit(1);
  }

  fprintf(stdprn,HP_INIT,PORTRAIT);

  if (add_adv_ZIP_check_digit(argv[1],data_buf,argv[2]))
    printf("Invalid ZIP code passed to function\n");
  else if (hp_printbarzip(25,30,data_buf))
    printf("Non-USPS ZIP character in string.\n");
  else
    printf("%s\n",data_buf);

  fprintf(stdprn,HP_DEINIT);
}

// H P _ I N I T
//
void hp_init()
{
  fprintf(stdprn,"\33&l0O\33(s0p10h12v0s0b3T");
}
 
// H P _ D E I N I T
//
void hp_deinit()
{
  fprintf(stdprn,"\33E");
}

// H P _ P R I N T B A R Z I P
//
int hp_printbarzip(int col, int row, char *code)
{
  char jstr[11];
  int i,j,pointx,pointy;

  hp_gotoxy(col,row,&pointx,&pointy);

  for (i=0 ; code[i] ; i++)
  {
    switch(code[i])
    {
      case '*': strcpy(jstr,"1-"); break;
      case '0': strcpy(jstr,"1-1-0-0-0-"); break;
      case '1': strcpy(jstr,"0-0-0-1-1-"); break;
      case '2': strcpy(jstr,"0-0-1-0-1-"); break;
      case '3': strcpy(jstr,"0-0-1-1-0-"); break;
      case '4': strcpy(jstr,"0-1-0-0-1-"); break;
      case '5': strcpy(jstr,"0-1-0-1-0-"); break;
      case '6': strcpy(jstr,"0-1-1-0-0-"); break;
      case '7': strcpy(jstr,"1-0-0-0-1-"); break;
      case '8': strcpy(jstr,"1-0-0-1-0-"); break;
      case '9': strcpy(jstr,"1-0-1-0-0-"); break;
      case '|': strcpy(jstr,"1-"); break;
      default:  return(1);
    }
    for (j=0 ; jstr[j] ; j++)
    {
      if (j % 2 == 0)
      {
        if (jstr[j] == '1')
          fprintf(stdprn,BAR,pointx,pointy,NARROW,TALL,BLACK);
        else
          fprintf(stdprn,BAR,pointx,pointy+SHORT_START,NARROW,SHORT,BLACK);
      }
      else
        fprintf(stdprn,BAR,pointx,pointy+SHORT_START,WIDE,SHORT,WHITE);
      pointx += 7;
    }
  } 
  return(0);
}

// H P _ G O T O X Y
//
void hp_gotoxy(int col, int row, int *px, int *py)
{
  fprintf(stdprn,"\33(8U\33(s0p%dh3T",CPI);
  fprintf(stdprn,"\33&a%dc%dR",col,row);
  *px = (int)((float)col/(float)CPI*DENSITY);
  *py = (int)((float)row/(float)RPI*DENSITY);
  fprintf(stdprn,"\33*p%dx%dY",*px,*py);
}

// A D D _ A D V _ Z I P _ C H E C K _ D I G I T
//
// The ZIP code should be passed as a string of solid numeric characters.
// It may be 5 or nine digits in length
//
int add_adv_ZIP_check_digit(char *zip_code, char *data_buf,char *addr)
{
  char *isspace,jzip[14];
  int  i, zip_sum=0, check_digit, adv_code=0;

  // TRUNCATE TRAILING SPACES.  EMBEDDED SPACES ARE ALSO BAD
  // SO WE USE strchr INSTEAD OF GOING TO THE END AND COUNTING BACK
  if ((isspace=strchr(zip_code,' ')) != NULL) // truncate trailing spaces
    *isspace = '\0';

  // CHECK TO SEE IF TRUNCATED ZIP IS WITHIN RANGE
  if (strlen(zip_code) < 5 || strlen(zip_code) > 9)
    return(1);

  // ONLY 9 DIGIT ZIP CAN TRY FOR THE 2 ADVANCED DIGITS
  if (strlen(zip_code) == 9)
  {
    for (i=0 ; !isdigit(addr[i]) ; i++);        // find number (if any else 0)
    if (i < strlen(addr))
    {
      adv_code = atoi(&addr[i]);                // make it an integer
      adv_code %= 100;                          // isolate last two digits
      sprintf(jzip,"%s%02d",zip_code,adv_code); // combine with zip code
    }
    else
      strcpy(jzip,zip_code);
  }
  else
    strcpy(jzip,zip_code);

  // CALCULATE THE CHECK DIGIT
  for (i = 0; jzip[i] ; i++)                  // sum all numbers
    zip_sum += jzip[i]-0x30;
  check_digit = zip_sum % 10 == 0 ? 0 : 10-(zip_sum % 10); // compute check

  // FORM THE COMPLETE ZIP BY ADDING THE START AND STOP CODES
  sprintf(data_buf,"*%s%d*",jzip,check_digit);// get it all together

  return(0);
}

